/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.uiengine.manager.impl;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.uiengine.config.UIComponent;
import org.tinygroup.uiengine.config.UIComponents;
import org.tinygroup.uiengine.manager.UIComponentManager;

public class UIComponentManagerImpl implements UIComponentManager {
	static Logger logger = LoggerFactory
			.getLogger(UIComponentManagerImpl.class);
	private TreeSet<UIComponent> uiComponentSet = new TreeSet<UIComponent>();
	private Map<String, UIComponent> uiMap = new HashMap<String, UIComponent>();

	public Set<UIComponent> getUiComponents() {
		return uiComponentSet;
	}

	public Set<UIComponent> getHealthUiComponents() {
		TreeSet<UIComponent> result = new TreeSet<UIComponent>();
		if (getUiComponents() != null) {
			for (UIComponent component : getUiComponents()) {
				if (isHealth(component.getName())) {
					result.add(component);
				}
			}
		}
		return result;
	}

	public void addUIComponents(UIComponents uiComponents) {
		uiComponentSet.addAll(uiComponents.getComponents());
		for(UIComponent component:uiComponents.getComponents()){
			uiMap.put(component.getName(), component);
		}
	}

	public boolean isHealth(String name) {
		UIComponent component = getUIComponent(name);
		if (component.isComputed()) {// 如果已经计算好，则直接返回
			return component.isHealth();
		} else {
			// 如果不依赖，则健康
			if (component.getDependencies() == null
					|| component.getDependencies().trim().length() == 0) {
				component.setComputed(true);
				component.setHealth(true);
				return true;
			} else {
				String[] dependencies = component.getDependencies().split(",");
				for (String dependencyName : dependencies) {
					if (!isHealth(dependencyName)) {
						logger.logMessage(
								LogLevel.ERROR,
								"UI包<{}>依赖的<{}>UI包，不能被找到，因此不会被加载！依赖此包的应用可能不会被顺序执行。",
								name, dependencyName);
						component.setComputed(true);
						component.setHealth(false);
						return false;
					}
				}
				return true;
			}
		}
	}

	public UIComponent getUIComponent(String name) {
		return uiMap.get(name);
	}
}
